@@ -2,7 +2,7 @@
 
     Minimum Profit - Programmer Text Editor
 
-    Copyright (C) 1991-2007 Angel Ortega <angel@triptico.com>
+    Copyright (C) 1991-2012 Angel Ortega <angel@triptico.com>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -22,18 +22,25 @@
 
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern int mp_exit_requested;
-extern mpdm_t mp;
+#define MP mpdm_hget_s(mpdm_root(), L"mp")
 
 mpdm_t mp_draw(mpdm_t doc, int optimize);
 
 mpdm_t mp_active(void);
-mpdm_t mp_process_action(mpdm_t action);
-mpdm_t mp_process_event(mpdm_t keycode);
-mpdm_t mp_set_y(mpdm_t doc, int y);
+void mp_process_action(mpdm_t action);
+void mp_process_event(mpdm_t keycode);
+void mp_set_y(mpdm_t doc, int y);
 mpdm_t mp_build_status_line(void);
 mpdm_t mp_get_history(mpdm_t key);
 mpdm_t mp_menu_label(mpdm_t action);
-mpdm_t mp_pending_key(void);
-mpdm_t mp_process_keyseq(mpdm_t key);
+mpdm_t mp_get_doc_names(void);
 int mp_keypress_throttle(int keydown);
+
+#ifdef __cplusplus
+}
+#endif
