@@ -1,9 +1,9 @@
 /*
 
     MPDM - Minimum Profit Data Manager
-    Copyright (C) 2003/2007 Angel Ortega <angel@triptico.com>
+    Copyright (C) 2003/2012 Angel Ortega <angel@triptico.com>
 
-    mpdm_t.c - Basic value management
+    mpdm_v.c - Basic value management
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-    http://www.triptico.com
+    http://triptico.com
 
 */
 
@@ -32,62 +32,76 @@
 
 #include "mpdm.h"
 
-/*******************
-	Data
-********************/
+
+/** data **/
 
 /* control structure */
 
 struct mpdm_control *mpdm = NULL;
 
-/*******************
-	Code
-********************/
 
-static void cleanup_value(mpdm_t v)
-/* cleans a value */
+/** code **/
+
+mpdm_t _mpdm_malloc(void)
 {
-	/* collapse multiple values */
-	if (v->flags & MPDM_MULTIPLE)
-		mpdm_collapse(v, 0, v->size);
-
-	/* free data if needed */
-	if (v->data != NULL && v->flags & MPDM_FREE) {
-		mpdm->memory_usage -= v->size;
-		free((void *)v->data);
-		v->data = NULL;
-	}
+    return malloc(sizeof(struct mpdm_val));
 }
 
 
-int mpdm_destroy(mpdm_t v)
+void _mpdm_free(mpdm_t v)
+{
+    free(v);
+}
+
+
+static void destroy_value(mpdm_t v)
 /* destroys a value */
 {
-	/* if still referenced or deleted, do nothing */
-	if (v->ref || v->flags & MPDM_DELETED)
-		return 0;
+    /* collapse multiple values */
+    if (v->flags & MPDM_MULTIPLE) {
+        int n;
 
-	/* dequeue */
-	v->next->prev = v->prev;
-	v->prev->next = v->next;
+        for (n = 0; n < mpdm_size(v); n++)
+            mpdm_unref(mpdm_aget(v, n));
+    }
 
-	/* if it's the current one, move to next */
-	if (mpdm->cur == v)
-		mpdm->cur = v->next;
+    /* free data if needed */
+    if (v->data != NULL && v->flags & MPDM_FREE) {
+        free((void *) v->data);
+        v->data = NULL;
+    }
 
-	/* account one value less */
-	mpdm->count--;
+    mpdm->count--;
 
-	/* add to the deleted values queue */
-	v->next = mpdm->del;
-	mpdm->del = v;
+    if (!(v->flags & MPDM_NONDYN))
+        _mpdm_free(v);
+}
 
-	cleanup_value(v);
 
-	/* mark as deleted */
-	v->flags |= MPDM_DELETED;
+/**
+ * mpdm_init - Initializes a value.
+ * @v: the value to initialize
+ * @flags: flags
+ * @data: pointer to real data
+ * @size: size of data
+ *
+ * Initializes a value.
+ *
+ * This function is normally not directly used; use any of the type
+ * creation macros instead.
+ * [Value Creation]
+ */
+mpdm_t mpdm_init(mpdm_t v, int flags, const void *data, int size)
+{
+    /* if v is NULL crash JUST NOW */
+    v->flags    = flags;
+    v->ref      = 0;
+    v->data     = data;
+    v->size     = size;
 
-	return 1;
+    mpdm->count++;
+
+    return v;
 }
 
 
@@ -108,40 +122,7 @@ int mpdm_destroy(mpdm_t v)
  */
 mpdm_t mpdm_new(int flags, const void *data, int size)
 {
-	mpdm_t v = NULL;
-
-	/* alloc */
-	if ((v = mpdm->del) != NULL)
-		mpdm->del = v->next;
-	else
-	if ((v = malloc(sizeof(struct mpdm_val))) == NULL)
-		return NULL;
-
-	/* add to the circular list */
-	if (mpdm->cur == NULL)
-		v->next = v->prev = v;
-	else {
-		v->prev = mpdm->cur;
-		v->next = mpdm->cur->next;
-
-		v->prev->next = v->next->prev = v;
-	}
-
-	mpdm->cur = v->next;
-
-	/* account one value more */
-	mpdm->count++;
-
-	/* count memory if data is dynamic */
-	if (flags & MPDM_FREE)
-		mpdm->memory_usage += size;
-
-	v->flags = flags;
-	v->ref = 0;
-	v->data = data;
-	v->size = size;
-
-	return v;
+    return mpdm_init(_mpdm_malloc(), flags & ~MPDM_NONDYN, data, size);
 }
 
 
@@ -154,9 +135,10 @@ mpdm_t mpdm_new(int flags, const void *data, int size)
  */
 mpdm_t mpdm_ref(mpdm_t v)
 {
-	if (v != NULL)
-		v->ref++;
-	return v;
+    if (v != NULL)
+        v->ref++;
+
+    return v;
 }
 
 
@@ -164,47 +146,39 @@ mpdm_t mpdm_ref(mpdm_t v)
  * mpdm_unref - Decrements the reference count of a value.
  * @v: the value
  *
- * Decrements the reference count of a value.
+ * Decrements the reference count of a value. If the reference
+ * count of the value reaches 0, it's destroyed.
  * [Value Management]
  */
 mpdm_t mpdm_unref(mpdm_t v)
 {
-	if (v != NULL)
-		v->ref--;
-	return v;
+    if (v != NULL) {
+        v->ref--;
+
+        if (v->ref <= 0) {
+            destroy_value(v);
+            v = NULL;
+        }
+    }
+
+    return v;
 }
 
 
 /**
- * mpdm_sweep - Sweeps unreferenced values.
- * @count: number of values to be swept
+ * mpdm_unrefnd - Decrements the reference count of a value, without destroy.
+ * @v: the value
  *
- * Destroys values with a reference count of 0. @count is the
- * number of values to be checked for deletion; special values of
- * @count are -1, that forces a check of all currently known values
- * (can be time-consuming) and 0, which tells mpdm_sweep() to check a
- * small group of them on each call.
+ * Decrements the reference count of a value, without destroying
+ * the value if it's unreferenced.
  * [Value Management]
  */
-void mpdm_sweep(int count)
+mpdm_t mpdm_unrefnd(mpdm_t v)
 {
-	/* if count is zero, sweep 'some' values */
-	if (count == 0) {
-		if (mpdm->default_sweep < 0)
-			count = mpdm->count / -mpdm->default_sweep;
-		else
-			count = mpdm->default_sweep;
-	}
+    if (v != NULL)
+        v->ref--;
 
-	/* if count is -1, sweep all */
-	if (count == -1)
-		count = mpdm->count;
-
-	for (; count > 0 && mpdm->count > mpdm->low_threshold; count--) {
-		/* destroy it or skip it */
-		if (!mpdm_destroy(mpdm->cur))
-			mpdm->cur = mpdm->cur->next;
-	}
+    return v;
 }
 
 
@@ -212,16 +186,19 @@ void mpdm_sweep(int count)
  * mpdm_size - Returns the size of an element.
  * @v: the element
  *
- * Returns the size of an element.
+ * Returns the size of an element. It does not change the
+ * reference count of the value.
  * [Value Management]
  */
 int mpdm_size(const mpdm_t v)
 {
-	/* NULL values have no size */
-	if (v == NULL)
-		return 0;
+    int r = 0;
 
-	return v->size;
+    /* NULL values have no size */
+    if (v != NULL)
+        r = v->size;
+
+    return r;
 }
 
 
@@ -236,10 +213,14 @@ int mpdm_size(const mpdm_t v)
  */
 mpdm_t mpdm_clone(const mpdm_t v)
 {
-	if (MPDM_IS_ARRAY(v))
-		return mpdm_aclone(v);
+    mpdm_t r;
 
-	return v;
+    if (MPDM_IS_ARRAY(v) && !MPDM_IS_EXEC(v))
+        r = mpdm_aclone(v);
+    else
+        r = v;
+
+    return r;
 }
 
 
@@ -252,30 +233,89 @@ mpdm_t mpdm_clone(const mpdm_t v)
  */
 mpdm_t mpdm_root(void)
 {
-	if (mpdm->root == NULL)
-		mpdm->root = mpdm_ref(MPDM_H(0));
+    if (mpdm->root == NULL)
+        mpdm->root = mpdm_ref(MPDM_H(0));
 
-	return mpdm->root;
+    return mpdm->root;
 }
 
 
+/**
+ * mpdm_set_ival - Sets the integer value.
+ * @v: the value
+ * @ival: the integer
+ *
+ * Sets the integer value for @v. It does not change
+ * the reference count of @v.
+ */
 mpdm_t mpdm_set_ival(mpdm_t v, int ival)
 /* sets an integer value to a value */
 {
-	v->flags |= MPDM_IVAL;
-	v->ival = ival;
+    v->flags    |= MPDM_IVAL;
+    v->ival     = ival;
 
-	return v;
+    return v;
 }
 
 
+/**
+ * mpdm_set_rval - Sets the real value.
+ * @v: the value
+ * @rval: the real
+ *
+ * Sets the real value for @v. It does not change
+ * the reference count of @v.
+ */
 mpdm_t mpdm_set_rval(mpdm_t v, double rval)
 /* sets a real value to a value */
 {
-	v->flags |= MPDM_RVAL;
-	v->rval = rval;
+    v->flags    |= MPDM_RVAL;
+    v->rval     = rval;
 
-	return v;
+    return v;
+}
+
+
+/**
+ * mpdm_void - Refs then unrefs a value.
+ * @v: the value
+ *
+ * References and unreferences a value. To be used to receive
+ * the output of mpdm_exec() in case of it being void (i.e.
+ * its return value ignored).
+ */
+void mpdm_void(mpdm_t v)
+{
+    mpdm_ref(v);
+    mpdm_unref(v);
+}
+
+
+/**
+ * mpdm_is_null - Returns 1 if a value is NULL.
+ * @v: the value
+ *
+ * Returns 1 if a value is NULL. The reference count is touched.
+ */
+int mpdm_is_null(mpdm_t v)
+{
+    int r;
+
+    mpdm_ref(v);
+    r = v == NULL ? 1 : 0;
+    mpdm_unref(v);
+
+    return r;
+}
+
+
+mpdm_t mpdm_set(mpdm_t *v, mpdm_t w)
+{
+    mpdm_ref(w);
+    mpdm_unref(*v);
+    *v = w;
+
+    return w;
 }
 
 
@@ -283,147 +323,180 @@ mpdm_t mpdm_set_rval(mpdm_t v, double rval)
  * mpdm_exec - Executes an executable value.
  * @c: the code value
  * @args: the arguments
+ * @ctxt: the context
  *
  * Executes an executable value. If @c is a scalar value, its data
  * should be a pointer to a directly executable C function with a
- * prototype of mpdm_t func(mpdm_t args); if it's a multiple one,
- * the first value's data should be a pointer to a directly executable C
- * function with a prototype of mpdm_t func(mpdm_t b, mpdm_t args) and
+ * prototype of mpdm_t func(mpdm_t args, mpdm_t ctxt); if it's a multiple
+ * one, the first value's data should be a pointer to a directly executable
+ * C function with a prototype of
+ * mpdm_t func(mpdm_t b, mpdm_t args, mpdm_t ctxt) and
  * the second value will be passed as the @b argument. This value is used
  * to store bytecode or so when implementing virtual machines or compilers.
+ * The @ctxt is meant to be used as a special context to implement local
+ * symbol tables and such. Its meaning is free and can be NULL.
  *
  * Returns the return value of the code. If @c is NULL or not executable,
  * returns NULL.
  * [Value Management]
  */
-mpdm_t mpdm_exec(mpdm_t c, mpdm_t args)
+mpdm_t mpdm_exec(mpdm_t c, mpdm_t args, mpdm_t ctxt)
 {
-	mpdm_t r = NULL;
+    mpdm_t r = NULL;
 
-	if (c != NULL && (c->flags & MPDM_EXEC)) {
-		mpdm_ref(c);
-		mpdm_ref(args);
+    mpdm_ref(c);
+    mpdm_ref(args);
+    mpdm_ref(ctxt);
 
-		if (c->flags & MPDM_MULTIPLE) {
-			mpdm_t x;
-			mpdm_t(*func) (mpdm_t, mpdm_t);
+    if (c != NULL && (c->flags & MPDM_EXEC)) {
 
-			/* value is multiple; first element is the
-			   2 argument version of the executable function,
-			   next its optional additional information and
-			   finally the arguments */
-			x = mpdm_aget(c, 0);
+        if (c->flags & MPDM_MULTIPLE) {
+            mpdm_t x;
+            mpdm_t(*func) (mpdm_t, mpdm_t, mpdm_t);
 
-			if ((func = (mpdm_t(*)(mpdm_t, mpdm_t)) (x->data)) != NULL)
-				r = func(mpdm_aget(c, 1), args);
-		}
-		else {
-			mpdm_t(*func) (mpdm_t);
+            /* value is multiple; first element is the
+               3 argument version of the executable function,
+               next its optional additional information,
+               the arguments and the context */
+            x = mpdm_aget(c, 0);
 
-			/* value is scalar; c is the 1 argument
-			   version of the executable function */
-			if ((func = (mpdm_t(*)(mpdm_t)) (c->data)) != NULL)
-				r = func(args);
-		}
+            if ((func =
+                 (mpdm_t(*)(mpdm_t, mpdm_t, mpdm_t)) (x->data)) != NULL)
+                r = func(mpdm_aget(c, 1), args, ctxt);
+        }
+        else {
+            mpdm_t(*func) (mpdm_t, mpdm_t);
 
-		mpdm_unref(args);
-		mpdm_unref(c);
-	}
+            /* value is scalar; c is the 2 argument
+               version of the executable function */
+            if ((func = (mpdm_t(*)(mpdm_t, mpdm_t)) (c->data)) != NULL)
+                r = func(args, ctxt);
+        }
+    }
 
-	return r;
+    mpdm_unref(ctxt);
+    mpdm_unref(args);
+    mpdm_unref(c);
+
+    return r;
 }
 
 
-mpdm_t mpdm_exec_1(mpdm_t c, mpdm_t a1)
+mpdm_t mpdm_exec_1(mpdm_t c, mpdm_t a1, mpdm_t ctxt)
 {
-	mpdm_t r;
-	mpdm_t a = MPDM_A(1);
+    mpdm_t r;
+    mpdm_t a = MPDM_AA(1);
 
-	mpdm_aset(a, a1, 0);
-	r = mpdm_exec(c, a);
-	mpdm_destroy(a);
-	return r;
+    mpdm_ref(a);
+    mpdm_aset(a, a1, 0);
+
+    r = mpdm_exec(c, a, ctxt);
+
+    mpdm_unref(a);
+
+    return r;
 }
 
 
-mpdm_t mpdm_exec_2(mpdm_t c, mpdm_t a1, mpdm_t a2)
+mpdm_t mpdm_exec_2(mpdm_t c, mpdm_t a1, mpdm_t a2, mpdm_t ctxt)
 {
-	mpdm_t r;
-	mpdm_t a = MPDM_A(2);
+    mpdm_t r;
+    mpdm_t a = MPDM_AA(2);
 
-	mpdm_aset(a, a1, 0);
-	mpdm_aset(a, a2, 1);
-	r = mpdm_exec(c, a);
-	mpdm_destroy(a);
-	return r;
+    mpdm_ref(a);
+    mpdm_aset(a, a1, 0);
+    mpdm_aset(a, a2, 1);
+
+    r = mpdm_exec(c, a, ctxt);
+
+    mpdm_unref(a);
+
+    return r;
 }
 
 
-mpdm_t mpdm_exec_3(mpdm_t c, mpdm_t a1, mpdm_t a2, mpdm_t a3)
+mpdm_t mpdm_exec_3(mpdm_t c, mpdm_t a1, mpdm_t a2, mpdm_t a3, mpdm_t ctxt)
 {
-	mpdm_t r;
-	mpdm_t a = MPDM_A(3);
+    mpdm_t r;
+    mpdm_t a = MPDM_AA(3);
 
-	mpdm_aset(a, a1, 0);
-	mpdm_aset(a, a2, 1);
-	mpdm_aset(a, a3, 2);
-	r = mpdm_exec(c, a);
-	mpdm_destroy(a);
-	return r;
+    mpdm_ref(a);
+    mpdm_aset(a, a1, 0);
+    mpdm_aset(a, a2, 1);
+    mpdm_aset(a, a3, 2);
+
+    r = mpdm_exec(c, a, ctxt);
+
+    mpdm_unref(a);
+
+    return r;
 }
 
 
-mpdm_t mpdm_xnew(mpdm_t(*a1) (mpdm_t, mpdm_t), mpdm_t a2)
+mpdm_t mpdm_xnew(mpdm_t(*a1) (mpdm_t, mpdm_t, mpdm_t), mpdm_t a2)
 {
-	mpdm_t x;
+    mpdm_t x;
 
-	x = MPDM_A(2);
-	x->flags |= MPDM_EXEC;
+    x = MPDM_A(2);
+    x->flags |= MPDM_EXEC;
 
-	mpdm_aset(x, MPDM_X(a1), 0);
-	mpdm_aset(x, a2, 1);
+    mpdm_ref(x);
 
-	return x;
+    mpdm_aset(x, MPDM_X(a1), 0);
+    mpdm_aset(x, a2, 1);
+
+    mpdm_unrefnd(x);
+
+    return x;
 }
 
 
-static mpdm_t MPDM(const mpdm_t args)
+mpdm_t mpdm_new_copy(int flags, void *ptr, int size)
+{
+    mpdm_t r = NULL;
+
+    if (ptr != NULL) {
+        char *ptr2 = malloc(size);
+        memcpy(ptr2, ptr, size);
+
+        r = mpdm_new(MPDM_FREE | flags, ptr2, size);
+    }
+
+    return r;
+}
+
+
+static mpdm_t MPDM(const mpdm_t args, mpdm_t ctxt)
 /* accesor / mutator for MPDM internal data */
 {
-	mpdm_t v = mpdm_aget(args, 0);
-	int n, c = 0;
+    mpdm_t v;
 
-	if (v != NULL) {
-		mpdm_t w;
+    mpdm_ref(args);
 
-		/* do changes */
-		if ((w = mpdm_hget_s(v, L"low_threshold")) != NULL && mpdm_ival(w) > 0)
-			mpdm->low_threshold = mpdm_ival(w);
+    v = mpdm_aget(args, 0);
 
-		if ((w = mpdm_hget_s(v, L"default_sweep")) != NULL)
-			mpdm->default_sweep = mpdm_ival(w);
+    if (v != NULL) {
+        mpdm_t w;
 
-		if ((w = mpdm_hget_s(v, L"hash_buckets")) != NULL)
-			mpdm->hash_buckets = mpdm_ival(w);
-	}
+        /* do changes */
+        if ((w = mpdm_hget_s(v, L"hash_buckets")) != NULL)
+            mpdm->hash_buckets = mpdm_ival(w);
+    }
 
-	/* loop all values counting the references ones */
-	for (n = mpdm->count, v = mpdm->cur; n > 0; n--, v = v->next)
-		if (v->ref == 0)
-			c++;
+    /* now collect all information */
+    v = MPDM_H(0);
 
-	/* now collect all information */
-	v = MPDM_H(0);
+    mpdm_ref(v);
 
-	mpdm_hset_s(v, L"version", MPDM_MBS(VERSION));
-	mpdm_hset_s(v, L"count", MPDM_I(mpdm->count));
-	mpdm_hset_s(v, L"low_threshold", MPDM_I(mpdm->low_threshold));
-	mpdm_hset_s(v, L"default_sweep", MPDM_I(mpdm->default_sweep));
-	mpdm_hset_s(v, L"memory_usage", MPDM_I(mpdm->memory_usage));
-	mpdm_hset_s(v, L"hash_buckets", MPDM_I(mpdm->hash_buckets));
-	mpdm_hset_s(v, L"unreferenced", MPDM_I(c));
+    mpdm_hset_s(v, L"version",          MPDM_MBS(VERSION));
+    mpdm_hset_s(v, L"count",            MPDM_I(mpdm->count));
+    mpdm_hset_s(v, L"hash_buckets",     MPDM_I(mpdm->hash_buckets));
 
-	return v;
+    mpdm_unref(args);
+
+    mpdm_unrefnd(v);
+
+    return v;
 }
 
 extern char **environ;
@@ -431,23 +504,27 @@ extern char **environ;
 static mpdm_t build_env(void)
 /* builds a hash with the environment */
 {
-	char **ptr;
-	mpdm_t e = MPDM_H(0);
+    char **ptr;
+    mpdm_t e = MPDM_H(0);
 
-	for (ptr = environ; *ptr != NULL; ptr++) {
-		char *eq = strchr(*ptr, '=');
+    mpdm_ref(e);
 
-		if (eq != NULL) {
-			mpdm_t k, v;
+    for (ptr = environ; *ptr != NULL; ptr++) {
+        char *eq = strchr(*ptr, '=');
 
-			k = MPDM_NMBS((*ptr), eq - (*ptr));
-			v = MPDM_MBS(eq + 1);
+        if (eq != NULL) {
+            mpdm_t k, v;
 
-			mpdm_hset(e, k, v);
-		}
-	}
+            k = MPDM_NMBS((*ptr), eq - (*ptr));
+            v = MPDM_MBS(eq + 1);
 
-	return e;
+            mpdm_hset(e, k, v);
+        }
+    }
+
+    mpdm_unrefnd(e);
+
+    return e;
 }
 
 
@@ -459,35 +536,33 @@ static mpdm_t build_env(void)
  */
 int mpdm_startup(void)
 {
-	/* do the startup only unless done beforehand */
-	if (mpdm == NULL) {
-		/* alloc space */
-		if ((mpdm = malloc(sizeof(struct mpdm_control))) == NULL)
-			return -1;
+    /* do the startup only unless done beforehand */
+    if (mpdm == NULL) {
+        /* alloc space */
+        if ((mpdm = malloc(sizeof(struct mpdm_control))) == NULL)
+            return -1;
 
-		/* cleans it */
-		memset(mpdm, '\0', sizeof(struct mpdm_control));
+        /* cleans it */
+        memset(mpdm, '\0', sizeof(struct mpdm_control));
 
-		/* sets the defaults */
-		mpdm->low_threshold = 16;
-		mpdm->default_sweep = -50000;
-		mpdm->hash_buckets = 31;
+        /* sets the defaults */
+        mpdm->hash_buckets = 31;
 
-		/* sets the locale */
-		if (setlocale(LC_ALL, "") == NULL)
-			setlocale(LC_ALL, "C");
+        /* sets the locale */
+        if (setlocale(LC_ALL, "") == NULL)
+            setlocale(LC_ALL, "C");
 
-		mpdm_encoding(NULL);
+        mpdm_encoding(NULL);
 
-		/* store the MPDM() function */
-		mpdm_hset_s(mpdm_root(), L"MPDM", MPDM_X(MPDM));
+        /* store the MPDM() function */
+        mpdm_hset_s(mpdm_root(), L"MPDM", MPDM_X(MPDM));
 
-		/* store the ENV hash */
-		mpdm_hset_s(mpdm_root(), L"ENV", build_env());
-	}
+        /* store the ENV hash */
+        mpdm_hset_s(mpdm_root(), L"ENV", build_env());
+    }
 
-	/* everything went OK */
-	return 0;
+    /* everything went OK */
+    return 0;
 }
 
 
@@ -498,7 +573,7 @@ int mpdm_startup(void)
  */
 void mpdm_shutdown(void)
 {
-	/* dummy, by now */
+    /* dummy, by now */
 }
 
 /**
@@ -511,6 +586,19 @@ void mpdm_shutdown(void)
 /** mpdm_t MPDM_A(int n); */
 /* ; */
 
+/**
+ * MPDM_AA - Creates an array value (using alloca()).
+ * @n: Number of elements
+ *
+ * Creates a new array value with @n elements on the local
+ * function's stack. These values are destroyed when the
+ * function exits. They should only be used for temporary
+ * issues.
+ * [Value Creation]
+ */
+/** mpdm_t MPDM_AA(int n); */
+/* ; */
+
 /**
  * MPDM_H - Creates a hash value.
  * @n: Number of buckets in the hash (0: use default)
@@ -534,6 +622,20 @@ void mpdm_shutdown(void)
 /** mpdm_t MPDM_LS(wchar_t * wcs); */
 /* ; */
 
+/**
+ * MPDM_AS - Creates a string value from a literal string using alloca().
+ * @wcs: the wide character string
+ *
+ * Creates a new string value from a literal, wide character string.
+ * A pointer to the string will be stored in the value (not a copy).
+ * These values are destroyed when the function exits. They should
+ * only be used for temporary issues and cannot be assigned to arrays
+ * nor hashes.
+ * [Value Creation]
+ */
+/** mpdm_t MPDM_AS(wchar_t * wcs); */
+/* ; */
+
 /**
  * MPDM_S - Creates a string value from a string.
  * @wcs: the wide character string
@@ -648,6 +750,19 @@ void mpdm_shutdown(void)
 /** mpdm_t MPDM_X(mpdm_t (* func)(mpdm_t args)); */
 /* ; */
 
+/**
+ * MPDM_C - Creates a new value with a copy of a buffer.
+ * @flags: additional flags
+ * @ptr: pointer to data
+ * @size: data size
+ *
+ * Create a new value with a copy of a buffer. The value will store a copy
+ * of @ptr and have the additional @flags.
+ * [Value Creation]
+ */
+/** mpdm_t MPDM_C(int flags, void *ptr, int size); */
+/* ; */
+
 /**
  * MPDM_IS_ARRAY - Tests if a value is an array.
  * @v: the value
