@@ -1,7 +1,7 @@
 /*
 
     MPSL - Minimum Profit Scripting Language
-    Copyright (C) 2003/2009 Angel Ortega <angel@triptico.com>
+    Copyright (C) 2003/2012 Angel Ortega <angel@triptico.com>
 
     mpsl_f.c - Minimum Profit Scripting Language Function Library
 
@@ -19,7 +19,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-    http://www.triptico.com
+    http://triptico.com
 
 */
 
@@ -36,6 +36,8 @@
 
 /** code **/
 
+#define F_ARGS mpdm_t a, mpdm_t l
+
 #define A(n) mpdm_aget(a, n)
 #define A0 A(0)
 #define A1 A(1)
@@ -63,8 +65,9 @@
  */
 /** integer = size(v); */
 /* ; */
-static mpdm_t F_size(mpdm_t a) {
-	return MPDM_I(mpdm_size(A0));
+static mpdm_t F_size(F_ARGS)
+{
+    return MPDM_I(mpdm_size(A0));
 }
 
 /**
@@ -77,8 +80,9 @@ static mpdm_t F_size(mpdm_t a) {
  * [Value Management]
  */
 /** v2 = clone(v); */
-static mpdm_t F_clone(mpdm_t a) {
-	return mpdm_clone(A0);
+static mpdm_t F_clone(F_ARGS)
+{
+    return mpdm_clone(A0);
 }
 
 /**
@@ -91,8 +95,10 @@ static mpdm_t F_clone(mpdm_t a) {
  * [Input-Output]
  */
 /** dump(v); */
-static mpdm_t F_dump(mpdm_t a) {
-	mpdm_dump(A0); return NULL;
+static mpdm_t F_dump(F_ARGS)
+{
+    mpdm_dump(A0);
+    return NULL;
 }
 
 /**
@@ -104,8 +110,9 @@ static mpdm_t F_dump(mpdm_t a) {
  * [Strings]
  */
 /** string = dumper(v); */
-static mpdm_t F_dumper(mpdm_t a) {
-	return mpdm_dumper(A0);
+static mpdm_t F_dumper(F_ARGS)
+{
+    return mpdm_dumper(A0);
 }
 
 /**
@@ -126,8 +133,9 @@ static mpdm_t F_dumper(mpdm_t a) {
  * [Arrays]
  */
 /** integer = cmp(v); */
-static mpdm_t F_cmp(mpdm_t a) {
-	return MPDM_I(mpdm_cmp(A0, A1));
+static mpdm_t F_cmp(F_ARGS)
+{
+    return MPDM_I(mpdm_cmp(A0, A1));
 }
 
 /**
@@ -139,8 +147,9 @@ static mpdm_t F_cmp(mpdm_t a) {
  * [Arrays]
  */
 /** bool = is_array(v); */
-static mpdm_t F_is_array(mpdm_t a) {
-	return mpsl_boolean(MPDM_IS_ARRAY(A0));
+static mpdm_t F_is_array(F_ARGS)
+{
+    return mpsl_boolean(MPDM_IS_ARRAY(A0));
 }
 
 /**
@@ -152,8 +161,9 @@ static mpdm_t F_is_array(mpdm_t a) {
  * [Hashes]
  */
 /** bool = is_hash(v); */
-static mpdm_t F_is_hash(mpdm_t a) {
-	return mpsl_boolean(MPDM_IS_HASH(A0));
+static mpdm_t F_is_hash(F_ARGS)
+{
+    return mpsl_boolean(MPDM_IS_HASH(A0));
 }
 
 /**
@@ -164,8 +174,9 @@ static mpdm_t F_is_hash(mpdm_t a) {
  * [Value Management]
  */
 /** bool = is_exec(v); */
-static mpdm_t F_is_exec(mpdm_t a) {
-	return mpsl_boolean(MPDM_IS_EXEC(A0));
+static mpdm_t F_is_exec(F_ARGS)
+{
+    return mpsl_boolean(MPDM_IS_EXEC(A0));
 }
 
 /**
@@ -189,8 +200,14 @@ static mpdm_t F_is_exec(mpdm_t a) {
  * [Strings]
  */
 /** array = splice(v, i, offset, del); */
-static mpdm_t F_splice(mpdm_t a) {
-	return mpdm_splice(A0,A1,IA2,IA3);
+static mpdm_t F_splice(F_ARGS)
+{
+    return mpdm_splice(A0, A1, IA2, IA3);
+}
+
+static mpdm_t F_slice(F_ARGS)
+{
+    return mpdm_slice(A0, IA1, IA2);
 }
 
 /**
@@ -204,8 +221,9 @@ static mpdm_t F_splice(mpdm_t a) {
  * [Arrays]
  */
 /** expand(a, offset, num); */
-static mpdm_t F_expand(mpdm_t a) {
-	return mpdm_expand(A0,IA1,IA2);
+static mpdm_t F_expand(F_ARGS)
+{
+    return mpdm_expand(A0, IA1, IA2);
 }
 
 /**
@@ -219,8 +237,9 @@ static mpdm_t F_expand(mpdm_t a) {
  * [Arrays]
  */
 /** collapse(a, offset, num); */
-static mpdm_t F_collapse(mpdm_t a) {
-	return mpdm_collapse(A0,IA1,IA2);
+static mpdm_t F_collapse(F_ARGS)
+{
+    return mpdm_collapse(A0, IA1, IA2);
 }
 
 /**
@@ -235,8 +254,9 @@ static mpdm_t F_collapse(mpdm_t a) {
  * [Arrays]
  */
 /** e = ins(a, e, offset); */
-static mpdm_t F_ins(mpdm_t a) {
-	return mpdm_ins(A0,A1,IA2);
+static mpdm_t F_ins(F_ARGS)
+{
+    return mpdm_ins(A0, A1, IA2);
 }
 
 /**
@@ -249,12 +269,13 @@ static mpdm_t F_ins(mpdm_t a) {
  * the end of the array (so a value of -1 means delete the
  * last element of the array).
  *
- * Returns the deleted element.
+ * Returns NULL (previous versions returned the deleted element).
  * [Arrays]
  */
 /** v = adel(a, offset); */
-static mpdm_t F_adel(mpdm_t a) {
-	return mpdm_adel(A0,IA1);
+static mpdm_t F_adel(F_ARGS)
+{
+    return mpdm_adel(A0, IA1);
 }
 
 /**
@@ -268,21 +289,35 @@ static mpdm_t F_adel(mpdm_t a) {
  * [Arrays]
  */
 /** v = shift(a); */
-static mpdm_t F_shift(mpdm_t a) {
-	return mpdm_shift(A0);
+static mpdm_t F_shift(F_ARGS)
+{
+    return mpdm_shift(A0);
 }
 
 /**
  * push - Pushes a value into an array.
  * @a: the array
- * @e: the value
+ * @arg1: first value
+ * @arg2: second value
+ * @argn: nth value
  *
- * Pushes a value into an array (i.e. inserts at the end).
+ * Pushes values into an array (i.e. inserts at the end).
+ * Returns the last element pushed.
  * [Arrays]
  */
-/** e = push(a, e); */
-static mpdm_t F_push(mpdm_t a) {
-	return mpdm_push(A0,A1);
+/** argn = push(a, arg1 [, arg2, ... argn]); */
+static mpdm_t F_push(F_ARGS)
+{
+    int n;
+    mpdm_t r = NULL;
+
+    for (n = 1; n < mpdm_size(a); n++) {
+        mpdm_unref(r);
+        r = mpdm_push(A0, A(n));
+        mpdm_ref(r);
+    }
+
+    return mpdm_unrefnd(r);
 }
 
 /**
@@ -294,8 +329,9 @@ static mpdm_t F_push(mpdm_t a) {
  * [Arrays]
  */
 /** v = pop(a); */
-static mpdm_t F_pop(mpdm_t a) {
-	return mpdm_pop(A0);
+static mpdm_t F_pop(F_ARGS)
+{
+    return mpdm_pop(A0);
 }
 
 /**
@@ -313,8 +349,9 @@ static mpdm_t F_pop(mpdm_t a) {
  * [Arrays]
  */
 /** v = queue(a, e, size); */
-static mpdm_t F_queue(mpdm_t a) {
-	return mpdm_queue(A0,A1,IA2);
+static mpdm_t F_queue(F_ARGS)
+{
+    return mpdm_queue(A0, A1, IA2);
 }
 
 /**
@@ -329,8 +366,9 @@ static mpdm_t F_queue(mpdm_t a) {
  * [Arrays]
  */
 /** integer = seek(a, k, step); */
-static mpdm_t F_seek(mpdm_t a) {
-	return MPDM_I(mpdm_seek(A0,A1,IA2));
+static mpdm_t F_seek(F_ARGS)
+{
+    return MPDM_I(mpdm_seek(A0, A1, IA2));
 }
 
 /**
@@ -350,14 +388,21 @@ static mpdm_t F_seek(mpdm_t a) {
  */
 /** array = sort(a); */
 /** array = sort(a, sorting_func); */
-static mpdm_t F_sort(mpdm_t a) {
-	return mpdm_sort_cb(A0,1,A1);
+static mpdm_t F_sort(F_ARGS)
+{
+    mpdm_t r, v;
+
+    v = mpdm_ref(A0);
+    r = mpdm_sort_cb(mpdm_clone(v), 1, A1);
+    mpdm_unref(v);
+
+    return r;
 }
 
 /**
  * split - Separates a string into an array of pieces.
- * @s: the separator
  * @v: the value to be separated
+ * @s: the separator
  *
  * Separates the @v string value into an array of pieces, using @s
  * as a separator.
@@ -369,23 +414,28 @@ static mpdm_t F_sort(mpdm_t a) {
  * [Arrays]
  * [Strings]
  */
-/** array = split(s, v); */
-static mpdm_t F_split(mpdm_t a) {
-	return mpdm_split(A0,A1);
+/** array = split(v, s); */
+static mpdm_t F_split(F_ARGS)
+{
+    return mpdm_split(A0, A1);
 }
 
 /**
- * join - Joins all elements of an array into one.
- * @s: joiner string
+ * join - Joins an array.
  * @a: array to be joined
+ * @s: joiner string or second array
  *
- * Joins all elements from @a into one string, using @s as a glue.
+ * If @s is a string or NULL, returns a new string with all elements
+ * in @a joined using @s. If @s is an array, it returns a new one
+ * containing all elements of @a followed by all elements of @s.
  * [Arrays]
  * [Strings]
  */
-/** string = join(s, a); */
-static mpdm_t F_join(mpdm_t a) {
-	return mpdm_join(A0,A1);
+/** string = join(a, joiner_str); */
+/** array = join(a1, a2); */
+static mpdm_t F_join(F_ARGS)
+{
+    return mpdm_join(A0, A1);
 }
 
 /**
@@ -396,8 +446,9 @@ static mpdm_t F_join(mpdm_t a) {
  * [Hashes]
  */
 /** integer = hsize(h); */
-static mpdm_t F_hsize(mpdm_t a) {
-	return MPDM_I(mpdm_hsize(A0));
+static mpdm_t F_hsize(F_ARGS)
+{
+    return MPDM_I(mpdm_hsize(A0));
 }
 
 /**
@@ -409,8 +460,9 @@ static mpdm_t F_hsize(mpdm_t a) {
  * [Hashes]
  */
 /** bool = exists(h, k); */
-static mpdm_t F_exists(mpdm_t a) {
-	return mpsl_boolean(mpdm_exists(A0, A1));
+static mpdm_t F_exists(F_ARGS)
+{
+    return mpsl_boolean(mpdm_exists(A0, A1));
 }
 
 /**
@@ -423,8 +475,9 @@ static mpdm_t F_exists(mpdm_t a) {
  * [Hashes]
  */
 /** v = hdel(h, k); */
-static mpdm_t F_hdel(mpdm_t a) {
-	return mpdm_hdel(A0, A1);
+static mpdm_t F_hdel(F_ARGS)
+{
+    return mpdm_hdel(A0, A1);
 }
 
 /**
@@ -436,8 +489,9 @@ static mpdm_t F_hdel(mpdm_t a) {
  * [Arrays]
  */
 /** array = keys(h); */
-static mpdm_t F_keys(mpdm_t a) {
-	return mpdm_keys(A0);
+static mpdm_t F_keys(F_ARGS)
+{
+    return mpdm_keys(A0);
 }
 
 /**
@@ -461,8 +515,9 @@ static mpdm_t F_keys(mpdm_t a) {
  * [Character Set Conversion]
  */
 /** fd = open(filename, mode); */
-static mpdm_t F_open(mpdm_t a) {
-	return mpdm_open(A0, A1);
+static mpdm_t F_open(F_ARGS)
+{
+    return mpdm_open(A0, A1);
 }
 
 /**
@@ -473,8 +528,9 @@ static mpdm_t F_open(mpdm_t a) {
  * [Input-Output]
  */
 /** close(fd); */
-static mpdm_t F_close(mpdm_t a) {
-	return mpdm_close(A0);
+static mpdm_t F_close(F_ARGS)
+{
+    return mpdm_close(A0);
 }
 
 /**
@@ -486,8 +542,9 @@ static mpdm_t F_close(mpdm_t a) {
  * [Character Set Conversion]
  */
 /** string = read(fd); */
-static mpdm_t F_read(mpdm_t a) {
-	return mpdm_read(A0);
+static mpdm_t F_read(F_ARGS)
+{
+    return mpdm_read(A0);
 }
 
 /**
@@ -499,8 +556,9 @@ static mpdm_t F_read(mpdm_t a) {
  * [Input-Output]
  */
 /** string = getchar(fd); */
-static mpdm_t F_getchar(mpdm_t a) {
-	return mpdm_getchar(A0);
+static mpdm_t F_getchar(F_ARGS)
+{
+    return mpdm_getchar(A0);
 }
 
 /**
@@ -511,12 +569,13 @@ static mpdm_t F_getchar(mpdm_t a) {
  * Writes the first character in @s into @fd. No charset
  * conversion is done.
  *
- * Returns @s or NULL if the writing failed.
+ * Returns the number of chars written (0 or 1).
  * [Input-Output]
  */
 /** s = putchar(fd, s); */
-static mpdm_t F_putchar(mpdm_t a) {
-	return mpdm_putchar(A0, A1);
+static mpdm_t F_putchar(F_ARGS)
+{
+    return MPDM_I(mpdm_putchar(A0, A1));
 }
 
 /**
@@ -532,20 +591,22 @@ static mpdm_t F_putchar(mpdm_t a) {
  * [Input-Output]
  */
 /** integer = fseek(fd, offset, whence); */
-static mpdm_t F_fseek(mpdm_t a) {
-	return MPDM_I(mpdm_fseek(A0, IA1, IA2));
+static mpdm_t F_fseek(F_ARGS)
+{
+    return MPDM_I(mpdm_fseek(A0, IA1, IA2));
 }
 
 /**
- * fseek - Returns the current file pointer.
+ * ftell - Returns the current file pointer.
  * @fd: the file descriptor
  *
  * Returns the position of the file pointer in @fd.
  * [Input-Output]
  */
 /** integer = ftell(fd); */
-static mpdm_t F_ftell(mpdm_t a) {
-	return MPDM_I(mpdm_ftell(A0));
+static mpdm_t F_ftell(F_ARGS)
+{
+    return MPDM_I(mpdm_ftell(A0));
 }
 
 /**
@@ -556,8 +617,23 @@ static mpdm_t F_ftell(mpdm_t a) {
  * [Input-Output]
  */
 /** bool = unlink(filename); */
-static mpdm_t F_unlink(mpdm_t a) {
-	return mpsl_boolean(mpdm_unlink(A0));
+static mpdm_t F_unlink(F_ARGS)
+{
+    return mpsl_boolean(mpdm_unlink(A0));
+}
+
+/**
+ * rename - Renames a file.
+ * @oldpath: old path
+ * @newpath: new path
+ *
+ * Renames a file.
+ * [Input-Output]
+ */
+/** bool = rename(oldpath, newpath); */
+static mpdm_t F_rename(F_ARGS)
+{
+    return mpsl_boolean(mpdm_rename(A0, A1));
 }
 
 /**
@@ -578,8 +654,9 @@ static mpdm_t F_unlink(mpdm_t a) {
  * [Input-Output]
  */
 /** array = stat(filename); */
-static mpdm_t F_stat(mpdm_t a) {
-	return mpdm_stat(A0);
+static mpdm_t F_stat(F_ARGS)
+{
+    return mpdm_stat(A0);
 }
 
 /**
@@ -591,8 +668,9 @@ static mpdm_t F_stat(mpdm_t a) {
  * [Input-Output]
  */
 /** integer = chmod(filename, perms); */
-static mpdm_t F_chmod(mpdm_t a) {
-	return MPDM_I(mpdm_chmod(A0,A1));
+static mpdm_t F_chmod(F_ARGS)
+{
+    return MPDM_I(mpdm_chmod(A0, A1));
 }
 
 /**
@@ -605,8 +683,9 @@ static mpdm_t F_chmod(mpdm_t a) {
  * [Input-Output]
  */
 /** integer = chown(filename, uid, gid); */
-static mpdm_t F_chown(mpdm_t a) {
-	return MPDM_I(mpdm_chown(A0,A1,A2));
+static mpdm_t F_chown(F_ARGS)
+{
+    return MPDM_I(mpdm_chown(A0, A1, A2));
 }
 
 /**
@@ -626,8 +705,9 @@ static mpdm_t F_chown(mpdm_t a) {
  * [Input-Output]
  */
 /** array = glob(spec, base); */
-static mpdm_t F_glob(mpdm_t a) {
-	return mpdm_glob(A0, A1);
+static mpdm_t F_glob(F_ARGS)
+{
+    return mpdm_glob(A0, A1);
 }
 
 /**
@@ -650,8 +730,9 @@ static mpdm_t F_glob(mpdm_t a) {
  * [Character Set Conversion]
  */
 /** integer = encoding(charset); */
-static mpdm_t F_encoding(mpdm_t a) {
-	return MPDM_I(mpdm_encoding(A0));
+static mpdm_t F_encoding(F_ARGS)
+{
+    return MPDM_I(mpdm_encoding(A0));
 }
 
 /**
@@ -667,8 +748,24 @@ static mpdm_t F_encoding(mpdm_t a) {
  * [Input-Output]
  */
 /** fd = popen(prg, mode); */
-static mpdm_t F_popen(mpdm_t a) {
-	return mpdm_popen(A0, A1);
+static mpdm_t F_popen(F_ARGS)
+{
+    return mpdm_popen(A0, A1);
+}
+
+/**
+ * popen2 - Opens a pipe and returns an array of two pipes.
+ * @prg: the program to pipe
+ *
+ * Opens a read-write pipe and returns an array of two descriptors,
+ * one for reading and one for writing. If @prg could not be piped to,
+ * returns NULL.
+ * [Input-Output]
+ */
+/** array = popen2(prg); */
+static mpdm_t F_popen2(F_ARGS)
+{
+    return mpdm_popen2(A0);
 }
 
 /**
@@ -679,15 +776,16 @@ static mpdm_t F_popen(mpdm_t a) {
  * [Input-Output]
  */
 /** pclose(fd); */
-static mpdm_t F_pclose(mpdm_t a) {
-	return mpdm_pclose(A0);
+static mpdm_t F_pclose(F_ARGS)
+{
+    return mpdm_pclose(A0);
 }
 
 /**
  * regex - Matches a regular expression.
+ * @v: the value to be matched
  * @r: the regular expression
  * @ra: an array of regular expressions
- * @v: the value to be matched
  * @offset: offset from the start of the value
  *
  * Matches a regular expression against a value. Valid flags are `i',
@@ -716,18 +814,19 @@ static mpdm_t F_pclose(mpdm_t a) {
  * is returned.
  * [Regular Expressions]
  */
-/** string = regex(r, v); */
-/** string = regex(r, v, offset); */
-/** array = regex(ra, v); */
+/** string = regex(v, r); */
+/** string = regex(v, r, offset); */
+/** array = regex(v, ra); */
 /** array = regex(); */
-static mpdm_t F_regex(mpdm_t a) {
-	return mpdm_regex(A0,A1,IA2);
+static mpdm_t F_regex(F_ARGS)
+{
+    return mpdm_regex(A0, A1, IA2);
 }
 
 /**
  * sregex - Matches and substitutes a regular expression.
- * @r: the regular expression
  * @v: the value to be matched
+ * @r: the regular expression
  * @s: the substitution string, hash or code
  * @offset: offset from the start of v
  *
@@ -752,11 +851,12 @@ static mpdm_t F_regex(mpdm_t a) {
  * were done.
  * [Regular Expressions]
  */
-/** string = sregex(r, v, s); */
-/** string = sregex(r, v, s, offset); */
+/** string = sregex(v, r, s); */
+/** string = sregex(v, r, s, offset); */
 /** integer = sregex(); */
-static mpdm_t F_sregex(mpdm_t a) {
-	return mpdm_sregex(A0,A1,A2,IA3);
+static mpdm_t F_sregex(F_ARGS)
+{
+    return mpdm_sregex(A0, A1, A2, IA3);
 }
 
 /**
@@ -774,8 +874,9 @@ static mpdm_t F_sregex(mpdm_t a) {
  * [Localization]
  */
 /** string = gettext(str); */
-static mpdm_t F_gettext(mpdm_t a) {
-	return mpdm_gettext(A0);
+static mpdm_t F_gettext(F_ARGS)
+{
+    return mpdm_gettext(A0);
 }
 
 /**
@@ -792,8 +893,9 @@ static mpdm_t F_gettext(mpdm_t a) {
  * [Localization]
  */
 /** bool = gettext_domain(dom, data); */
-static mpdm_t F_gettext_domain(mpdm_t a) {
-	return MPDM_I(mpdm_gettext_domain(A0, A1));
+static mpdm_t F_gettext_domain(F_ARGS)
+{
+    return MPDM_I(mpdm_gettext_domain(A0, A1));
 }
 
 /**
@@ -805,8 +907,11 @@ static mpdm_t F_gettext_domain(mpdm_t a) {
  * [Code Control]
  */
 /** load(source_file); */
-static mpdm_t F_load(mpdm_t a) {
-	return mpdm_exec(mpsl_compile_file(A0), NULL);
+static mpdm_t F_load(F_ARGS)
+{
+    return mpdm_exec(mpsl_compile_file(A0,
+                                       mpsl_get_symbol(MPDM_LS(L"INC"),
+                                                       l)), NULL, l);
 }
 
 /**
@@ -818,8 +923,9 @@ static mpdm_t F_load(mpdm_t a) {
  * [Code Control]
  */
 /** func = compile(source); */
-static mpdm_t F_compile(mpdm_t a) {
-	return mpsl_compile(A0);
+static mpdm_t F_compile(F_ARGS)
+{
+    return mpsl_compile(A0);
 }
 
 /**
@@ -829,26 +935,12 @@ static mpdm_t F_compile(mpdm_t a) {
  * Simulates an error. The @err error message is stored in the `ERROR'
  * global variable and an internal abort global flag is set, so no further
  * MPSL code can be executed until reset.
+ * [Code Control]
  */
 /** error(err); */
-static mpdm_t F_error(mpdm_t a) {
-	return mpsl_error(A0);
-}
-
-/**
- * sweep - Sweeps unreferenced values.
- * @count: number of values to be swept
- *
- * Destroys values with a reference count of 0. @count is the
- * number of values to be checked for deletion; special values of
- * @count are -1, that forces a check of all currently known values
- * (can be time-consuming) and 0, which tells sweep() to check a
- * small group of them on each call.
- * [Value Management]
- */
-/** sweep(count); */
-static mpdm_t F_sweep(mpdm_t a) {
-	mpdm_sweep(IA0); return NULL;
+static mpdm_t F_error(F_ARGS)
+{
+    return mpsl_error(A0);
 }
 
 /**
@@ -859,8 +951,9 @@ static mpdm_t F_sweep(mpdm_t a) {
  * [Strings]
  */
 /** string = uc(str); */
-static mpdm_t F_uc(mpdm_t a) {
-	return mpdm_ulc(A0, 1);
+static mpdm_t F_uc(F_ARGS)
+{
+    return mpdm_ulc(A0, 1);
 }
 
 /**
@@ -871,18 +964,21 @@ static mpdm_t F_uc(mpdm_t a) {
  * [Strings]
  */
 /** string = uc(str); */
-static mpdm_t F_lc(mpdm_t a) {
-	return mpdm_ulc(A0, 0);
+static mpdm_t F_lc(F_ARGS)
+{
+    return mpdm_ulc(A0, 0);
 }
 
 /**
  * time - Returns the current time.
  *
  * Returns the current time from the epoch (C library time()).
+ * [Time]
  */
 /** integer = time(); */
-static mpdm_t F_time(mpdm_t a) {
-	return MPDM_I(time(NULL));
+static mpdm_t F_time(F_ARGS)
+{
+    return MPDM_I(time(NULL));
 }
 
 /**
@@ -893,14 +989,27 @@ static mpdm_t F_time(mpdm_t a) {
  * [Input-Output]
  */
 /** integer = chdir(dir); */
-static mpdm_t F_chdir(mpdm_t a) {
-	return MPDM_I(mpdm_chdir(A0));
+static mpdm_t F_chdir(F_ARGS)
+{
+    return MPDM_I(mpdm_chdir(A0));
+}
+
+/**
+ * getcwd - Returns the current working directory
+ *
+ * Returns the current working directory
+ * [Input-Output]
+ */
+/** path = getcwd(); */
+static mpdm_t F_getcwd(F_ARGS)
+{
+    return mpdm_getcwd();
 }
 
 /**
  * sscanf - Extracts data like sscanf().
- * @fmt: the string format
  * @str: the string to be parsed
+ * @fmt: the string format
  * @offset: the character offset to start scanning
  *
  * Extracts data from a string using a special format pattern, very
@@ -914,10 +1023,11 @@ static mpdm_t F_chdir(mpdm_t a) {
  * position in the scanned string is returned as the value.
  * [Strings]
  */
-/** array = sscanf(fmt, str); */
-/** array = sscanf(fmt, str, offset); */
-static mpdm_t F_sscanf(mpdm_t a) {
-	return mpdm_sscanf(A0, A1, IA2);
+/** array = sscanf(str, fmt); */
+/** array = sscanf(str, fmt, offset); */
+static mpdm_t F_sscanf(F_ARGS)
+{
+    return mpdm_sscanf(A0, A1, IA2);
 }
 
 /**
@@ -935,14 +1045,18 @@ static mpdm_t F_sscanf(mpdm_t a) {
  * [Code Control]
  */
 /** v = eval(code, args); */
-static mpdm_t F_eval(mpdm_t a)
+static mpdm_t F_eval(F_ARGS)
 {
-	mpdm_t c;
+    mpdm_t r, c;
 
-	a = mpdm_clone(a);
-	c = mpdm_adel(a, 0);
+    a = mpdm_ref(mpdm_clone(a));
+    c = mpdm_shift(a);
 
-	return mpsl_eval(c, a);
+    r = mpsl_eval(c, a, l);
+
+    mpdm_unref(a);
+
+    return r;
 }
 
 
@@ -958,19 +1072,22 @@ static mpdm_t F_eval(mpdm_t a)
  * [Strings]
  */
 /** string = sprintf(fmt, arg1 [,arg2 ... argn]); */
-static mpdm_t F_sprintf(mpdm_t a)
+static mpdm_t F_sprintf(F_ARGS)
 {
-	mpdm_t f;
-	mpdm_t v;
+    mpdm_t f, v, r;
 
-	a = mpdm_clone(a);
-	f = mpdm_adel(a, 0);
+    a = mpdm_ref(mpdm_clone(a));
+    f = mpdm_shift(a);
 
-	/* if the first argument is an array, take it as the arguments */
-	if ((v = mpdm_aget(a, 0)) != NULL && MPDM_IS_ARRAY(v))
-		a = v;
+    /* if the first argument is an array, take it as the arguments */
+    if ((v = mpdm_aget(a, 0)) != NULL && MPDM_IS_ARRAY(v))
+        a = v;
 
-	return mpdm_sprintf(f, a);
+    r = mpdm_sprintf(f, a);
+
+    mpdm_unref(a);
+
+    return r;
 }
 
 
@@ -984,13 +1101,13 @@ static mpdm_t F_sprintf(mpdm_t a)
  * [Input-Output]
  */
 /** print(arg1 [,arg2 ... argn]); */
-static mpdm_t F_print(mpdm_t a)
+static mpdm_t F_print(F_ARGS)
 {
-	int n;
+    int n;
 
-	for (n = 0; n < mpdm_size(a); n++)
-		mpdm_write_wcs(stdout, mpdm_string(A(n)));
-	return NULL;
+    for (n = 0; n < mpdm_size(a); n++)
+        mpdm_write_wcs(stdout, mpdm_string(A(n)));
+    return NULL;
 }
 
 
@@ -1009,14 +1126,14 @@ static mpdm_t F_print(mpdm_t a)
  * [Character Set Conversion]
  */
 /** integer = write(fd, arg1 [,arg2 ... argn]); */
-static mpdm_t F_write(mpdm_t a)
+static mpdm_t F_write(F_ARGS)
 {
-	int n, r = 0;
+    int n, r = 0;
 
-	for (n = 1; n < mpdm_size(a); n++)
-		r += mpdm_write(A0, A(n));
+    for (n = 1; n < mpdm_size(a); n++)
+        r += mpdm_write(A0, A(n));
 
-	return MPDM_I(r);
+    return MPDM_I(r);
 }
 
 
@@ -1029,14 +1146,14 @@ static mpdm_t F_write(mpdm_t a)
  * [Strings]
  */
 /** string = chr(c); */
-static mpdm_t F_chr(mpdm_t a)
+static mpdm_t F_chr(F_ARGS)
 {
-	wchar_t tmp[2];
+    wchar_t tmp[2];
 
-	tmp[0] = (wchar_t) mpdm_ival(mpdm_aget(a, 0));
-	tmp[1] = L'\0';
+    tmp[0] = (wchar_t) mpdm_ival(mpdm_aget(a, 0));
+    tmp[1] = L'\0';
 
-	return MPDM_S(tmp);
+    return MPDM_S(tmp);
 }
 
 
@@ -1049,217 +1166,561 @@ static mpdm_t F_chr(mpdm_t a)
  * [Strings]
  */
 /** integer = ord(str); */
-static mpdm_t F_ord(mpdm_t a)
+static mpdm_t F_ord(F_ARGS)
 {
-	int ret = 0;
-	mpdm_t v = mpdm_aget(a, 0);
+    int ret = 0;
+    mpdm_t v = mpdm_aget(a, 0);
 
-	if (v != NULL) {
-		wchar_t * ptr = mpdm_string(v);
-		ret = (int) *ptr;
-	}
+    if (v != NULL) {
+        wchar_t *ptr = mpdm_string(v);
+        ret = (int) *ptr;
+    }
 
-	return MPDM_I(ret);
+    return MPDM_I(ret);
 }
 
 
 /**
- * map - Maps an array into another.
+ * map - Maps a multiple value to an array.
+ * @a: the value (array or hash)
  * @filter: the filter
- * @a: the array
  *
  * Returns a new array built by applying the @filter to all the
- * elements of the array @a. The filter can be an executable function
- * accepting one argument, in which case the return value of the function
- * will be used as the output element; @filt can also be a hash, in which
- * case the original element will be used as a key to the hash and the
- * associated value used as the output element.
+ * elements of the @a multiple value. The filter can be an executable
+ * function accepting one argument if @a is an array and two if @a is
+ * a hash, in which case the return value of the function will be used
+ * as the output element; @filter can also be a hash, in which
+ * case each array element or hash key will be used as a key to the
+ * hash and the associated value used as the output element.
  *
  * [Arrays]
+ * [Hashes]
  */
-/** array = map(filter, a); */
-static mpdm_t F_map(mpdm_t a)
+/** array = map(a, filter); */
+static mpdm_t F_map(F_ARGS)
 {
-	mpdm_t key = mpdm_aget(a, 0);
-	mpdm_t set = mpdm_aget(a, 1);
-	mpdm_t out;
+    mpdm_t set = mpdm_aget(a, 0);
+    mpdm_t fil = mpdm_aget(a, 1);
+    mpdm_t out = NULL;
+    int n = 0, i = 0;
+    mpdm_t k, v;
 
-	/* map NULL to NULL */
-	if (set == NULL)
-		return NULL;
+    if (set != NULL) {
+        out = mpdm_ref(
+            MPDM_A(MPDM_IS_HASH(set) ? mpdm_hsize(set) : mpdm_size(set))
+        );
 
-	out = mpdm_ref(MPDM_A(mpdm_size(set)));
+        if (MPDM_IS_EXEC(fil)) {
+            while (mpdm_iterator(set, &n, &k, &v)) {
+                if (MPDM_IS_HASH(set))
+                    mpdm_aset(out, mpdm_exec_2(fil, k, v, l), i++);
+                else
+                    mpdm_aset(out, mpdm_exec_2(fil, v, k, l), i++);
+            }
+        }
+        else
+        if (MPDM_IS_HASH(fil)) {
+            while (mpdm_iterator(set, &n, &k, &v)) {
+                mpdm_aset(out, mpdm_hget(fil, v), i++);
+            }
+        }
+    }
 
-	if (MPDM_IS_EXEC(key)) {
-		int n;
-
-		/* executes the code using the element as argument
-		   and stores the result in the output array */
-		for (n = 0; n < mpdm_size(set); n++)
-			mpdm_aset(out, mpdm_exec_1(key, mpdm_aget(set, n)), n);
-	}
-	else
-	if(MPDM_IS_HASH(key)) {
-		int n;
-
-		/* maps each value using the element as key */
-		for (n = 0; n < mpdm_size(set); n++)
-			mpdm_aset(out, mpdm_hget(key, mpdm_aget(set, n)), n);
-	}
-
-	return mpdm_unref(out);
+    return mpdm_unrefnd(out);
 }
 
 
 /**
- * grep - Greps inside an array.
+ * hmap - Maps a multiple value to a hash.
+ * @a: the value (array or hash)
  * @filter: the filter
- * @a: the array
  *
- * Greps inside an array and returns another one containing only the
+ * Returns a new hash built by applying the @filter to all the elements
+ * of the @a multiple value. The filter can be an executable function
+ * accepting one argument if @a is an array and two if @a is a hash,
+ * in which case the return value of the function is expected to be a
+ * two element array, with the 0th element to be the key and the 1st
+ * element the value for the new pair in the output hash.
+ *
+ * [Arrays]
+ * [Hashes]
+ */
+/** hash = hmap(a, filter); */
+static mpdm_t F_hmap(F_ARGS)
+{
+    mpdm_t set = mpdm_aget(a, 0);
+    mpdm_t fil = mpdm_aget(a, 1);
+    mpdm_t out = NULL;
+    int n = 0;
+    mpdm_t k, v;
+
+    if (set != NULL) {
+        out = mpdm_ref(MPDM_H(0));
+
+        if (MPDM_IS_EXEC(fil)) {
+            while (mpdm_iterator(set, &n, &k, &v)) {
+                mpdm_t w = mpdm_ref(mpdm_exec_2(fil, k, v, l));
+
+                if (MPDM_IS_ARRAY(w))
+                    mpdm_hset(out, mpdm_aget(w, 0), mpdm_aget(w, 1));
+
+                mpdm_unref(w);
+            }
+        }
+    }
+
+    return mpdm_unrefnd(out);
+}
+
+
+/**
+ * grep - Greps inside a multiple value.
+ * @a: the array or hash
+ * @filter: the filter
+ *
+ * Greps inside a multiple value and returns another one containing only the
  * elements that passed the filter. If @filter is a string, it's accepted
- * as a regular expression, which will be applied to each element.
- * If @filter is executable, it will be called with the element as its
- * only argument and its return value used as validation.
+ * as a regular expression, which will be applied to each element or hash
+ * key. If @filter is executable, it will be called with the element as
+ * its only argument if @a is an array or with two if @a is a hash,
+ * and its return value used as validation.
  *
- * The new array will contain all elements that passed the filter.
+ * The new value will have the same type as @a and will contain all
+ * elements that passed the filter.
  * [Arrays]
  * [Regular Expressions]
  */
 /** array = grep(filter, a); */
-static mpdm_t F_grep(mpdm_t a)
+static mpdm_t F_grep(F_ARGS)
 {
-	mpdm_t key = mpdm_aget(a, 0);
-	mpdm_t set = mpdm_aget(a, 1);
-	mpdm_t out = mpdm_ref(MPDM_A(0));
+    mpdm_t set = mpdm_aget(a, 0);
+    mpdm_t fil = mpdm_aget(a, 1);
+    mpdm_t out = NULL;
+    int n = 0;
+    mpdm_t k, v;
 
-	if (MPDM_IS_EXEC(key)) {
-		int n;
+    if (set != NULL) {
 
-		/* it's executable */
-		for (n = 0; n < mpdm_size(set); n++) {
-			mpdm_t v = mpdm_aget(set, n);
+        out = mpdm_ref(MPDM_IS_HASH(set) ? MPDM_H(0) : MPDM_A(0));
 
-			if (mpsl_is_true(mpdm_exec_1(key, v)))
-				mpdm_push(out, v);
-		}
-	}
-	else
-	if(key->flags & MPDM_STRING) {
-		int n;
+        if (MPDM_IS_EXEC(fil)) {
+            while (mpdm_iterator(set, &n, &k, &v)) {
+                mpdm_t w = mpdm_ref(mpdm_exec_2(fil, k, v, l));
 
-		/* it's a string; use it as a regular expression */
-		for (n = 0; n < mpdm_size(set); n++) {
-			mpdm_t v = mpdm_aget(set, n);
+                if (mpsl_is_true(w)) {
+                    if (MPDM_IS_HASH(out))
+                        mpdm_hset(out, k, v);
+                    else
+                        mpdm_push(out, v);
+                }
 
-			if (mpdm_regex(key, v, 0))
-				mpdm_push(out, v);
-		}
-	}
+                mpdm_unref(w);
+            }
+        }
+        else
+        if (MPDM_IS_STRING(fil)) {
+            while (mpdm_iterator(set, &n, &k, &v)) {
+                mpdm_t w = mpdm_ref(mpdm_regex(v, fil, 0));
 
-	return mpdm_size(mpdm_unref(out)) == 0 ? NULL : out;
+                if (w) {
+                    if (MPDM_IS_HASH(out))
+                        mpdm_hset(out, k, v);
+                    else
+                        mpdm_push(out, v);
+                }
+
+                mpdm_unref(w);
+            }
+        }
+    }
+
+    return mpdm_unrefnd(out);
 }
 
-static mpdm_t F_getenv(mpdm_t a)
+static mpdm_t F_getenv(F_ARGS)
 {
-	mpdm_t e = mpdm_hget_s(mpdm_root(), L"ENV");
+    mpdm_t e = mpdm_hget_s(mpdm_root(), L"ENV");
 
-	return mpdm_hget(e, mpdm_aget(a, 0));
+    return mpdm_hget(e, mpdm_aget(a, 0));
 }
 
-static mpdm_t F_bincall(mpdm_t a) {
-	return MPDM_X(mpdm_ival(mpdm_aget(a, 0)));
+static mpdm_t F_bincall(F_ARGS)
+{
+    void *func;
+    char *ptr = mpdm_wcstombs(mpdm_string(mpdm_aget(a, 0)), NULL);
+
+    sscanf(ptr, "%p", &func);
+    free(ptr);
+
+    return MPDM_X(func);
 }
 
+/* FIXME: for randomizing */
+int getpid(void);
+
+/**
+ * random - Returns a random value.
+ *
+ * Returns a random number from 0 to value - 1.
+ * [Miscellaneous]
+ */
+/** integer = random(value); */
+static mpdm_t F_random(F_ARGS)
+{
+    static unsigned int seed = 0;
+    int r = 0;
+    int range = mpdm_ival(mpdm_aget(a, 0));
+
+    if (range == 0 || seed == 0) {
+        seed = time(NULL) ^ getpid();
+
+        if (r)
+            r = seed % range;
+    }
+    else {
+        seed = (seed * 58321) + 11113;
+        r = (seed >> 16) % range;
+    }
+
+    return MPDM_I(r);
+};
+
+
+/**
+ * sleep - Sleeps a number of milliseconds.
+ *
+ * Sleeps a number of milliseconds.
+ * [Threading]
+ * [Time]
+ */
+/** sleep(msecs); */
+static mpdm_t F_sleep(F_ARGS)
+{
+    mpdm_sleep(mpdm_ival(mpdm_aget(a, 0)));
+
+    return NULL;
+}
+
+
+/**
+ * mutex - Returns a new mutex.
+ *
+ * Returns a new mutex.
+ * [Threading]
+ */
+/** var = mutex(); */
+static mpdm_t F_mutex(F_ARGS)
+{
+    return mpdm_new_mutex();
+}
+
+
+/**
+ * mutex_lock - Locks a mutex (possibly waiting).
+ * @mtx: the mutex
+ *
+ * Locks a mutex. If the mutex is already locked by
+ * another process, it waits until it's unlocked.
+ * [Threading]
+ */
+/** mutex_lock(mtx); */
+static mpdm_t F_mutex_lock(F_ARGS)
+{
+    mpdm_mutex_lock(A0);
+    return NULL;
+}
+
+
+/**
+ * mutex_unlock - Unlocks a mutex.
+ * @mtx: the mutex
+ *
+ * Unlocks a mutex.
+ * [Threading]
+ */
+/** mutex_unlock(mtx); */
+static mpdm_t F_mutex_unlock(F_ARGS)
+{
+    mpdm_mutex_unlock(A0);
+    return NULL;
+}
+
+
+/**
+ * semaphore - Returns a new semaphore.
+ * cnt: the initial count of the semaphore.
+ *
+ * Returns a new semaphore.
+ * [Threading]
+ */
+/** var = semaphore(cnt); */
+static mpdm_t F_semaphore(F_ARGS)
+{
+    return mpdm_new_semaphore(IA0);
+}
+
+
+/**
+ * semaphore_wait - Waits for a semaphore to be ready.
+ * @sem: the semaphore to wait onto
+ *
+ * Waits for the value of a semaphore to be > 0. If it's
+ * not, the thread waits until it is.
+ * [Threading]
+ */
+/** semaphore_wait(sem); */
+static mpdm_t F_semaphore_wait(F_ARGS)
+{
+    mpdm_semaphore_wait(A0);
+    return NULL;
+}
+
+
+/**
+ * semaphore_post - Increments the value of a semaphore.
+ * @sem: the semaphore to increment
+ *
+ * Increments by 1 the value of a semaphore.
+ * [Threading]
+ */
+/** semaphore_post(mtx); */
+static mpdm_t F_semaphore_post(F_ARGS)
+{
+    mpdm_semaphore_post(A0);
+    return NULL;
+}
+
+
+/**
+ * tr - Transliterates a string.
+ * @str: the string
+ * @from: set of characters to be replaced
+ * @to: set of characters to replace
+ *
+ * Transliterates @str to a new string with all characters from @from
+ * replaced by those in @to.
+ * [Threading]
+ */
+/** tr(str, from, to); */
+static mpdm_t F_tr(F_ARGS)
+{
+    return mpdm_tr(A0, A1, A2);
+}
+
+
+/**
+ * strftime - Formats the time as a string.
+ * @f: format string
+ * @t: seconds (from the Unix epoch)
+ *
+ * Returns as a string, formatted using the @f format string,
+ * the time @t given as argument (or now if @t is not given).
+ * See the local clib strftime() implementation for details
+ * on the format string.
+ * [Strings]
+ * [Time]
+ */
+/** string = strftime(@f); */
+/** string = strftime(@f, @t); */
+static mpdm_t F_strftime(F_ARGS)
+{
+    char *ptr;
+    time_t t;
+    char tmp[2048];
+    struct tm *tm;
+
+    ptr = mpdm_wcstombs(mpdm_string(A0), NULL);
+
+    if ((t = IA1) == 0)
+        t = time(NULL);
+
+    tm = localtime(&t);
+
+    strftime(tmp, sizeof(tmp) - 1, ptr, tm);
+
+    free(ptr);
+
+    return MPDM_MBS(tmp);
+}
+
+
+/**
+ * fmt - Formats one sprintf-like value.
+ * @f: format string
+ * @v: value
+ *
+ * Like sprintf(), but for only one value. It's a forward implementation
+ * from MPSL 3.x $ (format operator). Accepts sprintf() percent-formats
+ * plus j (JSON output) and t (brace-enclosed strftime() formats).
+ * [Strings]
+ */
+/** string = fmt(@f, @v); */
+static mpdm_t F_fmt(F_ARGS)
+{
+    return mpdm_fmt(A0, A1);
+}
+
+
+/**
+ * connect - Opens a client TCP/IP socket.
+ * @h: host name or ip
+ * @s: service or port number
+ *
+ * Opens a client TCP/IP socket to the @h host at @s service (or port).
+ * Returns NULL if the connection cannot be done or a file type value,
+ * that can be used with all file operation functions, including close().
+ * [Sockets]
+ * [Input-Output]
+ */
+/** f = connect(@h, @s); */
+static mpdm_t F_connect(F_ARGS)
+{
+    return mpdm_connect(A0, A1);
+}
+
+
+/**
+ * new - Creates a new object using another as its base.
+ * @c1: class / base object
+ * @c2: class / base object
+ * @cn: class / base object
+ *
+ * Creates a new object using as classes or base objects all the ones
+ * sent as arguments (assumed to be hashes).
+ * 
+ * [Object-oriented programming]
+ */
+/** o = new(c1 [, c2, ...cn]); */
+static mpdm_t F_new(F_ARGS)
+{
+    int n;
+    mpdm_t r = mpdm_ref(MPDM_H(0));
+
+    for (n = 0; n < mpdm_size(a); n++) {
+        mpdm_t w, k, v;
+        int m = 0;
+
+        w = mpdm_ref(A(n));
+
+        if (MPDM_IS_HASH(w)) {
+            while (mpdm_iterator(w, &m, &k, &v))
+                mpdm_hset(r, k, mpdm_clone(v));
+        }
+
+        mpdm_unref(w);
+    }
+
+    return mpdm_unrefnd(r);
+}
 
 static struct {
-	wchar_t * name;
-	mpdm_t (* func)(mpdm_t);
+    wchar_t *name;
+     mpdm_t(*func) (mpdm_t, mpdm_t);
 } mpsl_funcs[] = {
-	{ L"size",	F_size },
-	{ L"clone",	F_clone },
-	{ L"dump",	F_dump },
-	{ L"dumper",	F_dumper },
-	{ L"cmp",	F_cmp },
-	{ L"is_array",	F_is_array },
-	{ L"is_hash",	F_is_hash },
-	{ L"is_exec",	F_is_exec },
-	{ L"splice",	F_splice },
-	{ L"expand",	F_expand },
-	{ L"collapse",	F_collapse },
-	{ L"ins",	F_ins },
-	{ L"adel",	F_adel },
-	{ L"shift",	F_shift },
-	{ L"push",	F_push },
-	{ L"pop",	F_pop },
-	{ L"queue",	F_queue },
-	{ L"seek",	F_seek },
-	{ L"sort",	F_sort },
-	{ L"split",	F_split },
-	{ L"join",	F_join },
-	{ L"hsize",	F_hsize },
-	{ L"exists",	F_exists },
-	{ L"hdel",	F_hdel },
-	{ L"keys",	F_keys },
-	{ L"open",	F_open },
-	{ L"close",	F_close },
-	{ L"read",	F_read },
-	{ L"write",	F_write },
-	{ L"getchar",	F_getchar },
-	{ L"putchar",	F_putchar },
-	{ L"fseek",	F_fseek },
-	{ L"ftell",	F_ftell },
-	{ L"unlink",	F_unlink },
-	{ L"stat",	F_stat },
-	{ L"chmod",	F_chmod },
-	{ L"chown",	F_chown },
-	{ L"glob",	F_glob },
-	{ L"encoding",	F_encoding },
-	{ L"popen",	F_popen },
-	{ L"pclose",	F_pclose },
-	{ L"regex",	F_regex },
-	{ L"sregex",	F_sregex },
-	{ L"load",	F_load },
-	{ L"compile",	F_compile },
-	{ L"error",	F_error },
-	{ L"eval",	F_eval },
-	{ L"print",	F_print },
-	{ L"gettext",	F_gettext },
-	{ L"gettext_domain", F_gettext_domain },
-	{ L"sprintf",	F_sprintf },
-	{ L"sweep",	F_sweep },
-	{ L"chr",	F_chr },
-	{ L"ord",	F_ord },
-	{ L"map",	F_map },
-	{ L"grep",	F_grep },
-	{ L"getenv",	F_getenv },
-	{ L"uc",	F_uc },
-	{ L"lc",	F_lc },
-	{ L"time",	F_time },
-	{ L"chdir",	F_chdir },
-	{ L"sscanf",	F_sscanf },
-	{ L"bincall",	F_bincall },
-	{ NULL,		NULL }
+    { L"size",           F_size },
+    { L"clone",          F_clone },
+    { L"dump",           F_dump },
+    { L"dumper",         F_dumper },
+    { L"cmp",            F_cmp },
+    { L"is_array",       F_is_array },
+    { L"is_hash",        F_is_hash },
+    { L"is_exec",        F_is_exec },
+    { L"splice",         F_splice },
+    { L"expand",         F_expand },
+    { L"collapse",       F_collapse },
+    { L"ins",            F_ins },
+    { L"adel",           F_adel },
+    { L"shift",          F_shift },
+    { L"push",           F_push },
+    { L"pop",            F_pop },
+    { L"queue",          F_queue },
+    { L"seek",           F_seek },
+    { L"sort",           F_sort },
+    { L"split",          F_split },
+    { L"join",           F_join },
+    { L"hsize",          F_hsize },
+    { L"exists",         F_exists },
+    { L"hdel",           F_hdel },
+    { L"keys",           F_keys },
+    { L"open",           F_open },
+    { L"close",          F_close },
+    { L"read",           F_read },
+    { L"write",          F_write },
+    { L"getchar",        F_getchar },
+    { L"putchar",        F_putchar },
+    { L"fseek",          F_fseek },
+    { L"ftell",          F_ftell },
+    { L"unlink",         F_unlink },
+    { L"rename",         F_rename },
+    { L"stat",           F_stat },
+    { L"chmod",          F_chmod },
+    { L"chown",          F_chown },
+    { L"glob",           F_glob },
+    { L"encoding",       F_encoding },
+    { L"popen",          F_popen },
+    { L"popen2",         F_popen2 },
+    { L"pclose",         F_pclose },
+    { L"regex",          F_regex },
+    { L"sregex",         F_sregex },
+    { L"load",           F_load },
+    { L"compile",        F_compile },
+    { L"error",          F_error },
+    { L"eval",           F_eval },
+    { L"print",          F_print },
+    { L"gettext",        F_gettext },
+    { L"gettext_domain", F_gettext_domain },
+    { L"sprintf",        F_sprintf },
+    { L"chr",            F_chr },
+    { L"ord",            F_ord },
+    { L"map",            F_map },
+    { L"hmap",           F_hmap },
+    { L"grep",           F_grep },
+    { L"getenv",         F_getenv },
+    { L"uc",             F_uc },
+    { L"lc",             F_lc },
+    { L"time",           F_time },
+    { L"chdir",          F_chdir },
+    { L"getcwd",         F_getcwd },
+    { L"sscanf",         F_sscanf },
+    { L"bincall",        F_bincall },
+    { L"random",         F_random },
+    { L"sleep",          F_sleep },
+    { L"mutex",          F_mutex },
+    { L"mutex_lock",     F_mutex_lock },
+    { L"mutex_unlock",   F_mutex_unlock },
+    { L"semaphore",      F_semaphore },
+    { L"semaphore_wait", F_semaphore_wait },
+    { L"semaphore_post", F_semaphore_post },
+    { L"tr",             F_tr },
+    { L"strftime",       F_strftime },
+    { L"connect",        F_connect },
+    { L"new",            F_new },
+    { L"fmt",            F_fmt },
+    { L"slice",          F_slice },
+    { NULL,              NULL }
 };
 
 
 mpdm_t mpsl_build_funcs(void)
 /* build all functions */
 {
-	mpdm_t c;
-	int n;
+    mpdm_t c;
+    int n;
 
-	/* creates all the symbols in the CORE library */
-	c = MPDM_H(0);
-	for (n = 0; mpsl_funcs[n].name != NULL; n++) {
-		mpdm_t f = MPDM_S(mpsl_funcs[n].name);
-		mpdm_t x = MPDM_X(mpsl_funcs[n].func);
+    /* creates all the symbols in the CORE library */
+    c = MPDM_H(0);
 
-		mpdm_hset(mpdm_root(), f, x);
-		mpdm_hset(c, f, x);
-	}
+    mpdm_ref(c);
 
-	return c;
+    for (n = 0; mpsl_funcs[n].name != NULL; n++) {
+        mpdm_t f = MPDM_S(mpsl_funcs[n].name);
+        mpdm_t x = MPDM_X(mpsl_funcs[n].func);
+
+        mpdm_hset(mpdm_root(), f, x);
+        mpdm_hset(c, f, x);
+    }
+
+    mpdm_unrefnd(c);
+
+    return c;
 }
-
-
