@@ -1,7 +1,7 @@
 /*
 
     MPSL - Minimum Profit Scripting Language
-    Copyright (C) 2003/2007 Angel Ortega <angel@triptico.com>
+    Copyright (C) 2003/2009 Angel Ortega <angel@triptico.com>
 
     stress.c - Stress tests for MPSL.
 
@@ -42,12 +42,14 @@ int i_failed_msgs = 0;
 	Code
 ********************/
 
-void do_test(char *str, int ok)
+#define do_test(s, o) _do_test(s, o ,__LINE__)
+
+void _do_test(char *str, int ok, int line)
 {
 	char tmp[1024];
 
-	sprintf(tmp, "%s: %s\n", str, ok ? "OK!" : "*** Failed ***");
-	printf(tmp);
+	sprintf(tmp, "%s (line %d): %s\n", str, line, ok ? "OK!" : "*** Failed ***");
+	printf("%s", tmp);
 
 	tests++;
 
@@ -62,28 +64,31 @@ void do_test(char *str, int ok)
 
 void do_set(mpdm_t * v1, mpdm_t v2)
 {
+	mpdm_ref(v2);
 	mpdm_unref(*v1);
-	*v1 = mpdm_ref(v2);
+	*v1 = v2;
 }
 
 
-mpdm_t do_test_mpsl(char *code)
+#define do_test_mpsl(c) _do_test_mpsl(c, __LINE__)
+
+mpdm_t _do_test_mpsl(char *code, int line)
 {
 	static mpdm_t v = NULL;
 
 	do_set(&v, mpsl_compile(MPDM_MBS(code)));
 
 	printf("Compile: ");
-	do_test(code, v != NULL);
+	_do_test(code, v != NULL, line);
 	return (v);
 }
 
 
-mpdm_t do_test_mpsl_file(char *file)
+mpdm_t do_test_mpsl_file(char *file, mpdm_t inc)
 {
 	static mpdm_t v = NULL;
 
-	do_set(&v, mpsl_compile_file(MPDM_MBS(file)));
+	do_set(&v, mpsl_compile_file(MPDM_MBS(file), inc));
 
 	printf("Compile file: ");
 	do_test(file, v != NULL);
@@ -95,7 +100,7 @@ mpdm_t do_test_exec(mpdm_t x, mpdm_t a)
 {
 	static mpdm_t v = NULL;
 
-	do_set(&v, mpdm_exec(x, a));
+	do_set(&v, mpdm_exec(x, a, NULL));
 
 	return (v);
 }
@@ -205,12 +210,16 @@ void test_mpsl2(void)
 	do_test("mpsl calculator 5", mpdm_rval(v) == 10.0);
 
 	v = do_test_exec(do_test_mpsl("2 + 3 * 4;"), NULL);
+    mpdm_ref(v);
 	w = do_test_exec(do_test_mpsl("2 + (3 * 4);"), NULL);
 	do_test("mpsl calculator 6 (operator precedence)", mpdm_rval(v) == mpdm_rval(w));
+    mpdm_unref(v);
 
 	v = do_test_exec(do_test_mpsl("2 + 3 * 4;"), NULL);
+    mpdm_ref(v);
 	w = do_test_exec(do_test_mpsl("(2 + 3) * 4;"), NULL);
 	do_test("mpsl calculator 7 (operator precedence)", mpdm_rval(v) != mpdm_rval(w));
+    mpdm_unref(v);
 
 	v = do_test_mpsl("/* array */ [\"this\", \"one\", \"is\", 666, \"cool\"];");
 	v = do_test_exec(v, NULL);
@@ -578,15 +587,18 @@ void test_mpsl3(void)
 void test_mpsl_file(void)
 {
 	mpdm_t v;
+	mpdm_t inc;
 
-	mpsl_trace = 0;
+	mpsl_trap(NULL);
 
-	/* Create an INC array with the current directory */
-	mpdm_exec(mpsl_compile(MPDM_LS(L"INC = [ '.' ];")), NULL);
+	inc = mpdm_ref(MPDM_A(0));
+	mpdm_push(inc, MPDM_LS(L"."));
 
 	printf("Compiling from file:\n");
-	v = do_test_mpsl_file("test.mpsl");
+	v = do_test_mpsl_file("test.mpsl", inc);
 	v = do_test_exec(v, NULL);
+
+	mpdm_unref(inc);
 }
 
 
@@ -596,37 +608,38 @@ void test_abort_and_eval(void)
 	mpdm_t e;
 
 	mpsl_abort = 0;
-	v = mpsl_compile(MPDM_LS(L"1000;"));
-	do_test("Abort 1", mpdm_ival(mpdm_exec(v, NULL)) == 1000);
+	v = mpdm_ref(mpsl_compile(MPDM_LS(L"1000;")));
+	do_test("Abort 1", mpdm_ival(mpdm_exec(v, NULL, NULL)) == 1000);
 
 	/* set global abort function */
 	mpsl_abort = 1;
-	do_test("Abort 2", mpdm_exec(v, NULL) == NULL);
+	do_test("Abort 2", mpdm_exec(v, NULL, NULL) == NULL);
 
 	mpsl_abort = 0;
-	do_test("Abort 3", mpdm_ival(mpdm_exec(v, NULL)) == 1000);
+	do_test("Abort 3", mpdm_ival(mpdm_exec(v, NULL, NULL)) == 1000);
+	mpdm_unref(v);
 
 	mpsl_error(NULL);
 
-	v = mpsl_eval(MPDM_LS(L"invalid_code()"), NULL);
+	v = mpsl_eval(MPDM_LS(L"invalid_code()"), NULL, NULL);
 	e = mpdm_hget_s(mpdm_root(), L"ERROR");
 	printf("The following error is OK:\n");
 	mpdm_dump(e);
 	do_test("eval 1", v == NULL && e != NULL);
 
-	v = mpsl_eval(MPDM_LS(L"undef_func();"), NULL);
+	v = mpsl_eval(MPDM_LS(L"undef_func();"), NULL, NULL);
 	e = mpdm_hget_s(mpdm_root(), L"ERROR");
 	printf("The following error is also OK:\n");
 	mpdm_dump(e);
 	do_test("eval 2", v == NULL && e != NULL);
 
-	v = mpsl_eval(MPDM_LS(L"load('unexistent_file.mpsl');"), NULL);
+	v = mpsl_eval(MPDM_LS(L"load('unexistent_file.mpsl');"), NULL, NULL);
 	e = mpdm_hget_s(mpdm_root(), L"ERROR");
 	printf("The following error is also OK:\n");
 	mpdm_dump(e);
 	do_test("eval 3", v == NULL && e != NULL);
 
-	v = mpsl_eval(MPDM_LS(L"2000;"), NULL);
+	v = mpsl_eval(MPDM_LS(L"2000;"), NULL, NULL);
 	e = mpdm_hget_s(mpdm_root(), L"ERROR");
 	do_test("eval 4", mpdm_ival(v) == 2000 && e == NULL);
 }
@@ -636,27 +649,12 @@ int main(void)
 {
 	mpsl_startup();
 
-	mpdm->default_sweep = 64;
-
-	mpsl_trace = 1;
 	test_mpsl();
 	test_mpsl2();
 	test_mpsl3();
 	test_mpsl_file();
 	test_abort_and_eval();
 
-	printf("memory: %d\n", mpdm->memory_usage);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	mpdm_sweep(-1);
-	printf("memory: %d\n", mpdm->memory_usage);
-
 	mpsl_shutdown();
 
 	printf("\n*** Total tests passed: %d/%d\n", oks, tests);
