@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,37 +44,50 @@
      SYMBOL = 262,
      LITERAL = 263,
      WHILE = 264,
-     IF = 265,
-     SUB = 266,
-     FOREACH = 267,
-     LOCAL = 268,
-     BREAK = 269,
-     RETURN = 270,
-     IFI = 271,
-     ELSE = 272,
-     BOOLOR = 273,
-     BOOLAND = 274,
-     IMOD = 275,
-     IDIV = 276,
-     IMUL = 277,
-     ISUB = 278,
-     IADD = 279,
-     DEC = 280,
-     INC = 281,
-     RANGE = 282,
-     HASHPAIR = 283,
-     NUMLE = 284,
-     NUMGE = 285,
-     NUMNE = 286,
-     STRNE = 287,
-     NUMEQ = 288,
-     STREQ = 289,
-     STRCAT = 290,
-     BITAND = 291,
-     BITXOR = 292,
-     BITOR = 293,
-     MOD = 294,
-     UMINUS = 295
+     FOR = 265,
+     IF = 266,
+     SUB = 267,
+     FOREACH = 268,
+     LOCAL = 269,
+     GLOBAL = 270,
+     BREAK = 271,
+     RETURN = 272,
+     IFI = 273,
+     ELSE = 274,
+     BOOLOR = 275,
+     BOOLAND = 276,
+     IJOIN = 277,
+     ORASSIGN = 278,
+     ISHL = 279,
+     ISHR = 280,
+     IBITXOR = 281,
+     IBITOR = 282,
+     IBITAND = 283,
+     IMOD = 284,
+     IDIV = 285,
+     IMUL = 286,
+     ISUB = 287,
+     IADD = 288,
+     DEC = 289,
+     INC = 290,
+     RANGE = 291,
+     ARROW = 292,
+     NUMLE = 293,
+     NUMGE = 294,
+     NUMNE = 295,
+     STRNE = 296,
+     NUMEQ = 297,
+     STREQ = 298,
+     JOIN = 299,
+     AMPERSAND = 300,
+     BITXOR = 301,
+     BITOR = 302,
+     SHR = 303,
+     SHL = 304,
+     POW = 305,
+     MOD = 306,
+     INVCALL = 307,
+     UMINUS = 308
    };
 #endif
 /* Tokens.  */
@@ -87,37 +98,50 @@
 #define SYMBOL 262
 #define LITERAL 263
 #define WHILE 264
-#define IF 265
-#define SUB 266
-#define FOREACH 267
-#define LOCAL 268
-#define BREAK 269
-#define RETURN 270
-#define IFI 271
-#define ELSE 272
-#define BOOLOR 273
-#define BOOLAND 274
-#define IMOD 275
-#define IDIV 276
-#define IMUL 277
-#define ISUB 278
-#define IADD 279
-#define DEC 280
-#define INC 281
-#define RANGE 282
-#define HASHPAIR 283
-#define NUMLE 284
-#define NUMGE 285
-#define NUMNE 286
-#define STRNE 287
-#define NUMEQ 288
-#define STREQ 289
-#define STRCAT 290
-#define BITAND 291
-#define BITXOR 292
-#define BITOR 293
-#define MOD 294
-#define UMINUS 295
+#define FOR 265
+#define IF 266
+#define SUB 267
+#define FOREACH 268
+#define LOCAL 269
+#define GLOBAL 270
+#define BREAK 271
+#define RETURN 272
+#define IFI 273
+#define ELSE 274
+#define BOOLOR 275
+#define BOOLAND 276
+#define IJOIN 277
+#define ORASSIGN 278
+#define ISHL 279
+#define ISHR 280
+#define IBITXOR 281
+#define IBITOR 282
+#define IBITAND 283
+#define IMOD 284
+#define IDIV 285
+#define IMUL 286
+#define ISUB 287
+#define IADD 288
+#define DEC 289
+#define INC 290
+#define RANGE 291
+#define ARROW 292
+#define NUMLE 293
+#define NUMGE 294
+#define NUMNE 295
+#define STRNE 296
+#define NUMEQ 297
+#define STREQ 298
+#define JOIN 299
+#define AMPERSAND 300
+#define BITXOR 301
+#define BITOR 302
+#define SHR 303
+#define SHL 304
+#define POW 305
+#define MOD 306
+#define INVCALL 307
+#define UMINUS 308
 
 
 
@@ -126,16 +150,16 @@
 typedef union YYSTYPE
 {
 
-/* Line 1676 of yacc.c  */
-#line 88 "mpsl.y"
+/* Line 2068 of yacc.c  */
+#line 86 "mpsl.y"
 
 	mpdm_t v;	/* a simple value */
 	mpdm_t ins;	/* an 'instruction': [ opcode, args ] */
 
 
 
-/* Line 1676 of yacc.c  */
-#line 139 "y.tab.h"
+/* Line 2068 of yacc.c  */
+#line 163 "y.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
